@@ -31,26 +31,20 @@ class KURL;
 class KTorrentViewItem;
 class KPopupMenu;
 class KTorrentCore;
+class KTorrentViewMenu;
 class ScanDialog;
+class QString;
 
 namespace kt
 {
 	class TorrentInterface;
+	class Group;
 }
 
-
 using namespace bt;
 
 /**
- * This is the main view class for KTorrent.  Most of the non-menu,
- * non-toolbar, and non-statusbar (e.g., non frame) GUI code should go
- * here.
- *
- * This ktorrent uses an HTML component as an example.
- *
- * @short Main view
- * @author Joris Guisson <joris.guisson@gmail.com>
- * @version 0.1
+ * List view which shows information about torrents.
  */
 class KTorrentView : public KListView
 {
@@ -59,41 +53,86 @@ public:
 	/**
 	 * Default constructor
 	 */
-	KTorrentView(QWidget *parent, bool seed_view = false);
+	KTorrentView(QWidget *parent);
 
 	/**
 	 * Destructor
 	 */
 	virtual ~KTorrentView();
+	
+	/// Update the caption, so the correct number of running torrents is shown in the tab
+	void updateCaption();
+	
+	/// Get the current group
+	const kt::Group* getCurrentGroup() const {return current_group;}
 
 	/// Get the current TorrentInterface object
 	kt::TorrentInterface* getCurrentTC();
-
-	QCStringList getTorrentInfo(kt::TorrentInterface* tc);
-
-	/// Enable or disable the debug view
-	void setShowDebugView(bool yes);
 	
 	/// Save the views settings
-	void saveSettings();
+	void saveSettings(KConfig* cfg,int idx);
+	
+	/// Load the views settings
+	void loadSettings(KConfig* cfg,int idx);
+	
+	/**
+	 * Put the current selection in a list.
+	 * @param sel The list to put it in
+	 */
+	void getSelection(QValueList<kt::TorrentInterface*> & sel);
+	
+	/**
+	 * Add the current selection to a group. 
+	 * @param g The group
+	 */
+	void addSelectionToGroup(kt::Group* g);
+	
+	/**
+	 * Reimplemented for header context menu
+	 */
+	bool eventFilter(QObject* watched, QEvent* e);
+	
+	/**
+	 * Is column visible?
+	 */
+	bool columnVisible(int index);
+	
+	/**
+	 * Setup the view columns, based upon the current group
+	 * This will hide some columns for uploads only groups.
+	 */
+	void setupViewColumns();
 	
 public slots:
+	void setCurrentGroup(kt::Group* group);
 	void addTorrent(kt::TorrentInterface* tc);
 	void removeTorrent(kt::TorrentInterface* tc);
 	void update();
-	void torrentFinished(kt::TorrentInterface* tc);
 	void startDownloads();
 	void stopDownloads();
+	void startAllDownloads();
+	void stopAllDownloads();
 	void manualAnnounce();
 	void previewFiles();
 	void removeDownloads();
+	void removeDownloadsAndData();
 	void onSelectionChanged();
 	void queueSlot();
 	void checkDataIntegrity();
+	void removeFromGroup();
+	void showAddPeersWidget();
+	void openOutputDirectory();
+	void openTorXDirectory();
+	void setDownloadLocationSlot();
+	void dhtSlot();
+	void utPexSlot();
+	void speedLimits();
 
 private slots:
 	void onExecuted(QListViewItem* item);
 	void showContextMenu(KListView* ,QListViewItem* item,const QPoint & p);
+	void onColumnVisibilityChange(int);
+	void gsmItemActived(const QString & group);
 	
 	
 signals:
@@ -105,18 +144,29 @@ signals:
 	void viewChange(kt::TorrentInterface* tc);
 	void updateActions(bool can_start,bool can_stop,bool can_remove,bool can_scan);
 	void queue(kt::TorrentInterface* tc);
+	void needsDataCheck(kt::TorrentInterface* tc);
+	void updateGroupsSubMenu(KPopupMenu* gsm);
+	void groupsSubMenuItemActivated(KTorrentView* v,const QString & group);
 
 private:
 	bool acceptDrag(QDropEvent* event) const;
 	int getNumRunning();
-	void makeMenu();
+	bool startDownload(kt::TorrentInterface* tc);
+	void stopDownload(kt::TorrentInterface* tc);
+	void showStartError();
+	virtual QDragObject* dragObject();
+	void setupColumns();
+	void insertColumn(QString label, Qt::AlignmentFlags);
+	void columnHide(int index);
+	void columnShow(int index);	
 		
 private:
-	bool m_seedView;
 	QMap<kt::TorrentInterface*,KTorrentViewItem*> items;
-	bool show_debug_view;
-	KPopupMenu* menu;
-	int stop_id, start_id, remove_id, preview_id, announce_id, queue_id,scan_id;
+	KTorrentViewMenu* menu;
+	KPopupMenu* m_headerMenu;
+	kt::Group* current_group;
+	Uint32 running;
+	Uint32 total;
 };
 
 #endif // _KTORRENTVIEW_H_
