@@ -24,9 +24,9 @@
 #include <qpainter.h>
 #include <math.h>
 #include <interfaces/functions.h>
+#include "ktorrentview.h"
 #include "ktorrentviewitem.h"
 
-
 using namespace bt;
 using namespace kt;
 /*
@@ -63,7 +63,6 @@ static QColor StatusToColor(TorrentStatus s,const QColorGroup & cg)
 	{
 		case kt::SEEDING :
 		case kt::DOWNLOADING:
-		case kt::COMPLETE :
 		case kt::ALLOCATING_DISKSPACE :
 			return green;
 		case kt::STALLED:
@@ -74,17 +73,53 @@ static QColor StatusToColor(TorrentStatus s,const QColorGroup & cg)
 		case kt::NOT_STARTED :
 		case kt::STOPPED:
 		case kt::QUEUED:
+		case kt::DOWNLOAD_COMPLETE :
+		case kt::SEEDING_COMPLETE :
 		default:
 			return cg.text();
 	}
 	return cg.text();
 }
 
+static QColor ratioToColor(float ratio)
+{
+	QColor green(40,205,40);
+	return ratio > 0.8 ? green : Qt::red;
+}
+
+static double Percentage(const TorrentStats & s)
+{
+	if (s.bytes_left_to_download == 0)
+	{
+		return 100.0;
+	}
+	else
+	{
+		if (s.total_bytes_to_download == 0)
+		{
+			return 100.0;
+		}
+		else
+		{
+			double perc = 100.0 - ((double)s.bytes_left_to_download / s.total_bytes_to_download) * 100.0;
+			if (perc > 100.0)
+				perc = 100.0;
+			else if (perc > 99.9)
+				perc = 99.9;
+			else if (perc < 0.0)
+				perc = 0.0;
+			
+			return perc;
+		}
+	}
+}
+
 
 
 KTorrentViewItem::KTorrentViewItem(QListView* parent,TorrentInterface* tc)
 	: KListViewItem(parent),tc(tc)
 {
+	m_parent = (KTorrentView*) parent;
 	update();
 }
 
@@ -92,96 +127,151 @@ KTorrentViewItem::KTorrentViewItem(QListView* parent,TorrentInterface* tc)
 KTorrentViewItem::~KTorrentViewItem()
 {}
 
-void KTorrentViewItem::update()
+QCStringList KTorrentViewItem::getTorrentInfo(kt::TorrentInterface* tc)
 {
-	/*
-	addColumn(i18n("File"));
-	addColumn(i18n("Status"));
-	addColumn(i18n("Dowloaded"));
-	addColumn(i18n("Uploaded"));
-	addColumn(i18n("Down Speed"));
-	addColumn(i18n("Up Speed"));
-	addColumn(i18n("Time Left"));
-	addColumn(i18n("Peers"));
-	addColumn(i18n("% Complete"));
-	*/
-
+	QCStringList info;
 	const TorrentStats & s = tc->getStats();
-
-	setText(0,s.torrent_name);
-	setText(1,tc->statusToString());
-	Uint64 nb = s.bytes_downloaded > s.total_bytes ? s.total_bytes : s.bytes_downloaded;
-	setText(2,BytesToString(nb));
-	setText(3,BytesToString(s.total_bytes_to_download));
-	setText(4,BytesToString(s.bytes_uploaded));
+	info.append(s.torrent_name.local8Bit());
+	info.append(tc->statusToString().local8Bit());
+	info.append(BytesToString(s.bytes_downloaded).local8Bit());
+	info.append(BytesToString(s.total_bytes_to_download).local8Bit());
+	info.append(BytesToString(s.bytes_uploaded).local8Bit());
 	if (s.bytes_left_to_download == 0)
-		setText(5,KBytesPerSecToString(0));
+		info.append(KBytesPerSecToString(0).local8Bit());
 	else
-		setText(5,KBytesPerSecToString(s.download_rate / 1024.0));
-	setText(6,KBytesPerSecToString(s.upload_rate / 1024.0));
-  
-	KLocale* loc = KGlobal::locale();
+		info.append(KBytesPerSecToString(s.download_rate / 1024.0).local8Bit());
+	
+	info.append(KBytesPerSecToString(s.upload_rate / 1024.0).local8Bit());
 	if (s.bytes_left_to_download == 0)
 	{
-		setText(7,i18n("finished"));
-		eta = -1;
+		info.append(QCString(""));
 	}
 	else if (s.running) 
 	{
-		float bytes_downloaded = (float)s.bytes_downloaded;
-		if( bytes_downloaded < 1 ) //if we just started download use old algorithm
+		Uint32 secs = tc->getETA();
+		if(secs == -1)
+			info.append(i18n("infinity").local8Bit());
+		else
+			info.append(DurationToString(secs).local8Bit());
+	}
+	else
+	{
+		info.append(i18n("infinity").local8Bit());
+	}
+	
+	info.append(QString::number(s.num_peers).local8Bit());
+	info.append(QString(KGlobal::locale()->formatNumber(Percentage(s),2) + " %").local8Bit());
+	info.append(KGlobal::locale()->formatNumber(kt::ShareRatio(s),2).local8Bit());
+	info.append(QString::number(s.seeders_connected_to).local8Bit());
+	info.append(QString::number(s.leechers_connected_to).local8Bit());
+	return info;
+}
+
+void KTorrentViewItem::update()
+{
+	const TorrentStats & s = tc->getStats();
+	
+	if(m_parent->columnVisible(0))
+		setText(0,s.torrent_name);
+				
+	if(m_parent->columnVisible(1))
+		setText(1,tc->statusToString());
+	
+	if(m_parent->columnVisible(2))
+	{
+		Uint64 nb = /*s.bytes_downloaded > s.total_bytes ? s.total_bytes : */s.bytes_downloaded;
+		setText(2,BytesToString(nb));
+	}
+	
+	if(m_parent->columnVisible(3))
+		setText(3,BytesToString(s.total_bytes_to_download));
+	
+	if(m_parent->columnVisible(4))
+		setText(4,BytesToString(s.bytes_uploaded));
+	
+	if(m_parent->columnVisible(5))
+	{
+		if (s.download_rate >= 103) // lowest "visible" speed, all below will be 0,0 Kb/s
 		{
-			if (s.download_rate == 0)
+			if (s.bytes_left_to_download == 0)
+				setText(5,KBytesPerSecToString(0));
+			else
+				setText(5,KBytesPerSecToString(s.download_rate / 1024.0));
+		}
+		else
+			setText(5, "");
+	}
+
+	if(m_parent->columnVisible(6))
+	{
+		if (s.upload_rate >= 103) // lowest "visible" speed, all below will be 0,0 Kb/s
+			setText(6,KBytesPerSecToString(s.upload_rate / 1024.0));
+		else
+			setText(6, "");
+	}
+
+	if(m_parent->columnVisible(7))
+	{
+		if (s.bytes_left_to_download == 0)
+		{
+			setText(7,QString::null);
+			eta = -1;
+		}
+		else if (s.running) 
+		{
+			Uint32 secs = tc->getETA();
+			if(secs == -1)
 			{
-				setText(7,i18n("infinity"));
+				setText(7,QString("%1").arg(QChar(0x221E)));
 				eta = -2;
 			}
 			else
 			{
-				Uint32 secs = (int)floor( (float)s.bytes_left_to_download / (float)s.download_rate);
 				eta = secs;
 				setText(7,DurationToString(secs));
-			}
-		}
-		else 
-		{
-			double avg_speed = (double)bytes_downloaded / (double)tc->getRunningTimeDL();
-			eta = (Int64)floor(s.bytes_left_to_download / avg_speed);
-			setText(7,DurationToString((int)floor(s.bytes_left_to_download / avg_speed)));
-		}
-	}
-	else
-	{
-		setText(7,i18n("infinity"));
-		eta = -2;
-	}
-	
-	setText(8,QString::number(s.num_peers));
-
-	double perc = 0;
-	if (s.bytes_left_to_download == 0)
-	{
-		perc = 100.0;
-	}
-	else
-	{
-		if (s.total_bytes_to_download == 0)
-		{
-			perc = 100.0;
+			}			
 		}
 		else
 		{
-			perc = 100.0 - ((double)s.bytes_left_to_download / s.total_bytes_to_download) * 100.0;
-			if (perc > 100.0)
-				perc = 100.0;
-			else if (perc > 99.9)
-				perc = 99.9;
-			else if (perc < 0.0)
-				perc = 0.0;
-				
+			setText(7,QString("%1").arg(QChar(0x221E)));
+			eta = -2;
 		}
 	}
-	setText(9,i18n("%1 %").arg(loc->formatNumber(perc,2)));
+	if(m_parent->columnVisible(8))	
+		setText(8,QString::number(s.num_peers));
+	
+	if(m_parent->columnVisible(8))
+	{
+		setText(8,QString("%1 (%2)").arg(QString::number(s.seeders_connected_to)).arg(QString::number(s.seeders_total)));	
+	}
+
+	if(m_parent->columnVisible(9))
+	{
+		setText(9,QString("%1 (%2)").arg(QString::number(s.leechers_connected_to)).arg(QString::number(s.leechers_total)));
+	}
+
+	if(m_parent->columnVisible(10))
+	{
+		setText(10,i18n("%1 %").arg(KGlobal::locale()->formatNumber(Percentage(s),2)));
+	}
+	
+	if(m_parent->columnVisible(11))
+	{
+		float ratio = kt::ShareRatio(s);
+		setText(11,QString("%1").arg(KGlobal::locale()->formatNumber(ratio,2)));
+	}
+	
+	if (m_parent->columnVisible(12))
+	{
+		Uint32 secs = tc->getRunningTimeDL();
+		setText(12,secs > 0 ? DurationToString(secs) : "");
+	}
+	
+	if (m_parent->columnVisible(13))
+	{
+		Uint32 secs = tc->getRunningTimeUL() - tc->getRunningTimeDL();
+		setText(13,secs > 0 ? DurationToString(secs) : "");
+	}
 }
 
 
@@ -216,8 +306,9 @@ int KTorrentViewItem::compare(QListViewItem * i,int col,bool) const
 				return -1;
 			else
 				return CompareVal(eta,other->eta);
-		case 8: return CompareVal(s.num_peers,os.num_peers);
-		case 9:
+		case 8: return CompareVal(s.seeders_total,os.seeders_total);
+		case 9: return CompareVal(s.leechers_total,os.leechers_total);
+		case 10:
 		{
 			double perc = s.total_bytes_to_download == 0 ? 100.0 : ((double)s.bytes_downloaded / s.total_bytes_to_download) * 100.0;
 			if (perc > 100.0)
@@ -227,6 +318,20 @@ int KTorrentViewItem::compare(QListViewItem * i,int col,bool) const
 				operc = 100.0;
 			return CompareVal(perc,operc);
 		}
+		case 11:
+		{
+			float r1 = kt::ShareRatio(s);
+			float r2 = kt::ShareRatio(os);
+			return CompareVal(r1,r2);
+		}
+		case 12:
+			return CompareVal(tc->getRunningTimeDL(),otc->getRunningTimeDL());
+		case 13:
+		{
+			Uint32 t = tc->getRunningTimeUL() - tc->getRunningTimeDL();
+			Uint32 ot = otc->getRunningTimeUL() - otc->getRunningTimeDL();
+			return CompareVal(t,ot);
+		}
 	}
 
 	return 0;
@@ -240,8 +345,10 @@ void KTorrentViewItem::paintCell(QPainter* p,const QColorGroup & cg,
 
 	if (column == 1)
 		_cg.setColor(QColorGroup::Text, StatusToColor(tc->getStats().status,cg));
+	
+	if (column == 11)
+		_cg.setColor(QColorGroup::Text, ratioToColor(kt::ShareRatio(tc->getStats())));
 
 
 	KListViewItem::paintCell(p,_cg,column,width,align);
 }
-
