@@ -24,9 +24,10 @@ namespace kt
 
 	Plugin::Plugin(QObject *parent, const char* qt_name,const QStringList & /*args*/,
 				   const QString & name,const QString & author,
-				   const QString & email,const QString & description)
+				   const QString & email,const QString & description,
+				   const QString & icon)
 	: KParts::Plugin(parent,qt_name),
-	name(name),author(author),email(email),description(description)
+	name(name),author(author),email(email),description(description),icon(icon)
 	{
 		core = 0;
 		gui = 0;
@@ -40,5 +41,9 @@ namespace kt
 	void Plugin::guiUpdate()
 	{
 	}
+	
+	void Plugin::shutdown(bt::WaitJob* )
+	{
+	}
 }
 #include "plugin.moc"
