@@ -20,8 +20,14 @@
 #ifndef KTPLUGIN_H
 #define KTPLUGIN_H
 
+#include <ktversion.h>
 #include <kparts/plugin.h>
 
+namespace bt
+{
+	class WaitJob;
+}
+
 namespace kt
 {
 	class CoreInterface;
@@ -50,10 +56,12 @@ namespace kt
 		 * @param author Author of plugin
 		 * @param mail E-mail address of author
 		 * @param description What does the plugin do
+		 * @param icon Name of the plugin's icon
 		 */
 		Plugin(QObject *parent,const char* qt_name,const QStringList & args,
 			   const QString & name,const QString & author,
-			   const QString & email,const QString & description);
+			   const QString & email,const QString & description,
+			   const QString & icon);
 		virtual ~Plugin();
 
 		/**
@@ -61,7 +69,7 @@ namespace kt
 		 * Any changes made here must be later made undone, when unload is
 		 * called.
 		 * Upon error a bt::Error should be thrown. And the plugin should remain
-		 * in an unitialized state. The Error contains an error message, which will
+		 * in an uninitialized state. The Error contains an error message, which will
 		 * get show to the user.
 		 */
 		virtual void load() = 0;
@@ -77,11 +85,19 @@ namespace kt
 		 * GUI updates.
 		 */
 		virtual void guiUpdate();
+		
+		/**
+		 * This should be implemented by plugins who need finish of some stuff which might take some time.
+		 * These operations must be finished or killed by a timeout before we can proceed with unloading the plugin. 
+		 * @param job The WaitJob which monitors the plugin
+		 */
+		virtual void shutdown(bt::WaitJob* job);
 
 		const QString & getName() const {return name;}
 		const QString & getAuthor() const {return author;}
 		const QString & getEMailAddress() const {return email;}
 		const QString & getDescription() const {return description;}
+		const QString & getIcon() const {return icon;}
 
 		/// Get a pointer to the CoreInterface
 		CoreInterface* getCore() {return core;}
@@ -112,11 +128,15 @@ namespace kt
 		/// See if the plugin is loaded
 		bool isLoaded() const {return loaded;}
 		
+		/// Check wether the plugin matches the version of KT
+		virtual bool versionCheck(const QString & version) const = 0;
+		
 	private:
 		QString name;
 		QString author;
 		QString email;
 		QString description;
+		QString icon;
 		CoreInterface* core;
 		GUIInterface* gui;
 		bool loaded;
