@@ -89,12 +89,12 @@ namespace dht
 		Out(SYS_DHT|LOG_NOTICE) << "DHT: Stopping " << endl;
 		srv->stop();
 		node->saveTable(table_file);
+		running = false;
+		stopped();
 		delete tman; tman = 0;
 		delete db; db = 0;
 		delete node; node = 0;
 		delete srv; srv = 0;
-		running = false;
-		stopped();
 	}
 
 	void DHT::ping(PingReq* r)
@@ -102,6 +102,10 @@ namespace dht
 		if (!running)
 			return;
 		
+		// ignore requests we get from ourself
+		if (r->getID() == node->getOurID())
+			return;
+		
 		Out(SYS_DHT|LOG_NOTICE) << "DHT: Sending ping response" << endl;
 		PingRsp rsp(r->getMTID(),node->getOurID());
 		rsp.setOrigin(r->getOrigin());
@@ -116,6 +120,10 @@ namespace dht
 		if (!running)
 			return;
 		
+		// ignore requests we get from ourself
+		if (r->getID() == node->getOurID())
+			return;
+		
 		Out(SYS_DHT|LOG_DEBUG) << "DHT: got findNode request" << endl;
 		node->recieved(this,r);
 		// find the K closest nodes and pack them
@@ -141,6 +149,10 @@ namespace dht
 		if (!running)
 			return;
 		
+		// ignore requests we get from ourself
+		if (r->getID() == node->getOurID())
+			return;
+		
 		Out(SYS_DHT|LOG_DEBUG) << "DHT: got announce request" << endl;
 		node->recieved(this,r);
 		// first check if the token is OK
@@ -165,6 +177,10 @@ namespace dht
 		if (!running)
 			return;
 		
+		// ignore requests we get from ourself
+		if (r->getID() == node->getOurID())
+			return;
+		
 		Out(SYS_DHT|LOG_DEBUG) << "DHT: got getPeers request" << endl;
 		node->recieved(this,r);
 		DBItemList dbl;
