@@ -20,6 +20,7 @@
 #include "torrent.h"
 #include "chunk.h"
 #include "cache.h"
+#include "peermanager.h"
 #include <util/functions.h>
 
 namespace bt
@@ -46,4 +47,9 @@ namespace bt
 	{
 		tmpdir = ndir;
 	}
+	
+	bool Cache::mappedModeAllowed()
+	{
+		return MaxOpenFiles() - bt::PeerManager::getTotalConnections() < 100;
+	}
 }
