@@ -58,12 +58,19 @@ namespace bt
 		virtual QString getOutputPath() const = 0;
 		
 		/**
-		 * Changes the tmp dir. All data files should allready been moved.
+		 * Changes the tmp dir. All data files should already been moved.
 		 * This just modifies the tmpdir variable.
 		 * @param ndir The new tmpdir
 		 */
 		virtual void changeTmpDir(const QString & ndir);
 		
+		/**
+		 * Changes output path. All data files should already been moved.
+		 * This just modifies the datadir variable.
+		 * @param outputpath New output path
+		 */
+		virtual void changeOutputPath(const QString & outputpath) = 0;
+		
 		/**
 		 * Load a chunk into memory. If something goes wrong,
 		 * an Error should be thrown.
@@ -124,6 +131,17 @@ namespace bt
 		 * empty directories should also be deleted.
 		 */
 		virtual void deleteDataFiles() = 0;
+		
+		/** 
+		 * See if we are allowed to use mmap, when loading chunks.
+		 * This will return false if we are close to system limits.
+		 */
+		static bool mappedModeAllowed();
+		
+		/**
+		 * Get the number of bytes all the files of this torrent are currently using on disk.
+		 * */
+		virtual Uint64 diskUsage() = 0;
 	};
 
 }
