@@ -20,7 +20,7 @@
 #ifndef BTCHUNKMANAGER_H
 #define BTCHUNKMANAGER_H
 
-#include <qvaluelist.h>
+#include <qmap.h>
 #include <qstring.h>
 #include <qobject.h>
 #include <qptrvector.h> 
@@ -47,7 +47,7 @@ namespace bt
 	 * @author Joris Guisson
 	 * 
 	 * Manages all Chunk's and the cache file, where all the chunk's are stored.
-	 * It also manages a seperate index file, where the position of each piece
+	 * It also manages a separate index file, where the position of each piece
 	 * in the cache file is stored.
 	 *
 	 * The chunks are stored in the cache file in the correct order. Eliminating
@@ -61,11 +61,16 @@ namespace bt
 		QString index_file,file_info_file,file_priority_file;
 		QPtrVector<Chunk> chunks;
 		Cache* cache;
-		QValueList<Uint32> loaded;
-		BitSet bitset,excluded_chunks;
+		QMap<Uint32,TimeStamp> loaded; // loaded chunks and when they were loaded
+		BitSet bitset;
+		BitSet excluded_chunks;
+		BitSet only_seed_chunks;
+		BitSet todo;
 		mutable Uint32 chunks_left;
 		mutable bool recalc_chunks_left;
-	
+		Uint32 corrupted_count;
+		Uint32 recheck_counter;
+		bool during_load;	
 	public:
 		ChunkManager(Torrent & tor,
 					 const QString & tmpdir,
@@ -82,6 +87,8 @@ namespace bt
 		/// Get the actual output path
 		QString getOutputPath() const;
 		
+		void changeOutputPath(const QString& output_path);
+		
 		/// Remove obsolete chunks
 		void checkMemoryUsage();
 		
@@ -99,9 +106,10 @@ namespace bt
 		
 		/**
 		 * Create the cache file, and index files.
+		 * @param check_priority Make sure chunk priorities and dnd status of files match
 		 * @throw Error When it can be created
 		 */
-		void createFiles();
+		void createFiles(bool check_priority = false);
 		
 		/**
 		 * Test all files and see if they are not missing.
@@ -145,7 +153,7 @@ namespace bt
 		/**
 		 * Prepare a chunk for downloading
 		 * @param c The Chunk
-		 * @param allways Allways do this, even if the chunk is not NOT_DOWNLOADED
+		 * @param allways Always do this, even if the chunk is not NOT_DOWNLOADED
 		 * @return true if ok, false if the chunk is not NOT_DOWNLOADED
 		 */
 		bool prepareChunk(Chunk* c,bool allways = false);
@@ -167,18 +175,23 @@ namespace bt
 		/**
 		 * Save the i'th Chunk to the cache_file.
 		 * Also changes the Chunk's status to ON_DISK.
-		 * The Chunk's data is immediatly cleared.
+		 * The Chunk's data is immediately cleared.
 		 * @param i The Chunk's index
 		 * @param update_index Update the index or not
 		 */
 		void saveChunk(unsigned int i,bool update_index = true);
 		
 		/**
-		 * Calculates the number of bytes left to download. Does include 
+		 * Calculates the number of bytes left for the tracker. Does include 
 		 * excluded chunks (this should be used for the tracker).
 		 * @return The number of bytes to download + the number of bytes excluded
 		 */
 		Uint64 bytesLeft() const;
+		
+		/**
+		 * Calculates the number of bytes left to download.
+		 */
+		Uint64 bytesLeftToDownload() const;
 
 		/**
 		 * Calculates the number of bytes which have been excluded.
@@ -206,15 +219,31 @@ namespace bt
 		 */
 		Uint32 chunksExcluded() const;
 		
+		/**
+		 * Get the number of downloaded chunks
+		 * @return 
+		 */
+		Uint32 chunksDownloaded() const;
+		
+		/**
+		 * Get the number of only seed chunks.
+		 */
+		Uint32 onlySeedChunks() const;
+		
 		/**
 		 * Get a BitSet of the status of all Chunks
 		 */
 		const BitSet & getBitSet() const {return bitset;}
 
 		/**
-		 * Get a BitSet of the status of all Chunks
+		 * Get the excluded bitset
 		 */
 		const BitSet & getExcludedBitSet() const {return excluded_chunks;}
+		
+		/**
+		 * Get the only seed bitset. 
+		 */
+		const BitSet & getOnlySeedBitSet() const {return only_seed_chunks;}
 
 		/// Get the number of chunks into the file.
 		Uint32 getNumChunks() const {return chunks.count();}
@@ -244,7 +273,7 @@ namespace bt
 		 */
 		void include(Uint32 from,Uint32 to);
 		
-		
+				
 		/**
 		 * Data has been checked, and these chunks are OK.
 		 * The ChunkManager will update it's internal structures
@@ -263,6 +292,15 @@ namespace bt
 		
 		/// Delete all data files 
 		void deleteDataFiles();
+		
+		/// Are all not deselected chunks downloaded.
+		bool completed() const;
+		
+		/// Set the maximum chunk size for a data check, 0 means alllways check
+		static void setMaxChunkSizeForDataCheck(Uint32 mcs) {max_chunk_size_for_data_check = mcs;}
+
+		/// Get the current disk usage of all the files in this torrent
+		Uint64 diskUsage();
 	signals:
 		/**
 		 * Emitted when a range of chunks has been excluded
@@ -284,6 +322,12 @@ namespace bt
 		 */
 		void updateStats();
 		
+		/**
+		 * A corrupted chunk has been found during uploading.
+		 * @param chunk The chunk
+		 */
+		void corrupted(Uint32 chunk);
+		
 	private:
 		void saveIndexFile();
 		void writeIndexFileEntry(Chunk* c);
@@ -294,7 +338,9 @@ namespace bt
 
 	private slots:
 		void downloadStatusChanged(TorrentFile* tf,bool download);
-		void downloadPriorityChanged(TorrentFile* tf,Priority newpriority);
+		void downloadPriorityChanged(TorrentFile* tf,Priority newpriority,Priority oldpriority);
+		
+		static Uint32 max_chunk_size_for_data_check;
 	};
 
 }
