@@ -139,10 +139,53 @@ namespace bt
 		}
 	}
 
-	void MultiFileCache::changeDataDir(const QString& ndir)
+	void MultiFileCache::changeTmpDir(const QString& ndir)
 	{
 		Cache::changeTmpDir(ndir);
 		cache_dir = tmpdir + "cache/";
+		QString dnd_dir = tmpdir + "dnd" + bt::DirSeparator();
+		
+		// change paths for individual files, it should not
+		// be a problem to move these files when they are open
+		for (Uint32 i = 0;i < tor.getNumFiles();i++)
+		{
+			TorrentFile & tf = tor.getFile(i);
+			if (tf.doNotDownload())
+			{
+				DNDFile* dfd = dnd_files.find(i);
+				if (dfd)
+					dfd->changePath(dnd_dir + tf.getPath() + ".dnd");
+			}
+			else
+			{
+				CacheFile* fd = files.find(i);
+				if (fd)
+					fd->changePath(cache_dir + tf.getPath());
+			}
+		}
+	}
+	
+	void MultiFileCache::changeOutputPath(const QString & outputpath)
+	{
+		output_dir = outputpath;
+		if (!output_dir.endsWith(bt::DirSeparator()))
+			output_dir += bt::DirSeparator();
+		
+		datadir = output_dir;
+		
+		if (!bt::Exists(cache_dir))
+			MakeDir(cache_dir);
+		
+		for (Uint32 i = 0;i < tor.getNumFiles();i++)
+		{
+			TorrentFile & tf = tor.getFile(i);
+			if (!tf.doNotDownload())
+			{
+				QString fpath = tf.getPath();
+				bt::Delete(cache_dir + fpath,true); // delete any existing symlinks
+				bt::SymLink(output_dir + fpath,cache_dir + fpath,true); // create new one
+			}
+		}
 	}
 
 	void MultiFileCache::create()
@@ -154,8 +197,9 @@ namespace bt
 		if (!bt::Exists(tmpdir + "dnd"))
 			bt::MakeDir(tmpdir + "dnd");
 
+		// update symlinks
 		for (Uint32 i = 0;i < tor.getNumFiles();i++)
-		{
+		{			
 			TorrentFile & tf = tor.getFile(i);
 			touch(tf);
 		}
@@ -165,7 +209,7 @@ namespace bt
 	{
 		QString fpath = tf.getPath();
 		bool dnd = tf.doNotDownload();
-		// first split fpath by / seperator
+		// first split fpath by / separator 
 		QStringList sl = QStringList::split(bt::DirSeparator(),fpath);
 		// create all necessary subdirs
 		QString ctmp = cache_dir;
@@ -228,11 +272,18 @@ namespace bt
 			if (!fd)
 				return;
 			
-			Uint64 off = FileOffset(c,f,tor.getChunkSize());
-			Uint8* buf = (Uint8*)fd->map(c,off,c->getSize(),CacheFile::READ);
-			if (buf)
-				c->setData(buf,Chunk::MMAPPED);
-			return;
+			if (Cache::mappedModeAllowed())
+			{
+				Uint64 off = FileOffset(c,f,tor.getChunkSize());
+				Uint8* buf = (Uint8*)fd->map(c,off,c->getSize(),CacheFile::READ);
+				if (buf)
+				{
+					c->setData(buf,Chunk::MMAPPED);
+					// only return when the mapping is OK
+					// if mmap fails we will just load it buffered
+					return;
+				}
+			}
 		}
 		
 		Uint8* data = new Uint8[c->getSize()];
@@ -297,12 +348,12 @@ namespace bt
 			Uint64 off = FileOffset(c,tor.getFile(tflist.first()),tor.getChunkSize());
 			CacheFile* fd = files.find(tflist.first());
 			Uint8* buf = 0;
-			if (fd)
+			if (fd && Cache::mappedModeAllowed())
 				buf = (Uint8*)fd->map(c,off,c->getSize(),CacheFile::RW);
+			
 			if (!buf)
 			{
-				// if mmap fails use buffered mode
-				Out() << "Warning : mmap failed, falling back to buffered mode" << endl;
+				// if mmap fails or is not possible use buffered mode
 				c->allocate();
 				c->setStatus(Chunk::BUFFERED);
 			}
@@ -337,24 +388,7 @@ namespace bt
 			c->setStatus(Chunk::ON_DISK);
 			return;
 		}
-	/*	else if (tflist.count() == 1 && c->getStatus() == Chunk::BUFFERED)
-		{
-			// buffered chunks are slightly more difficult
-			CacheFile* fd = files.find(tflist[0]);
-			if (!fd)
-				return;
-			
-			const TorrentFile & f = tor.getFile(tflist[0]);
-			
-			Uint64 off = FileOffset(c,f,tor.getChunkSize()); 
-					//c->getIndex() * tor.getChunkSize();
-			fd->write(c->getData(),c->getSize(),off);
-			c->clear();
-			c->setStatus(Chunk::ON_DISK);
-			return;
-		}
-	*/
-		
+	
 	//	Out() << "Writing to " << tflist.count() << " files " << endl;
 		Uint64 written = 0; // number of bytes written
 		for (Uint32 i = 0;i < tflist.count();i++)
@@ -404,49 +438,7 @@ namespace bt
 		c->clear();
 		c->setStatus(Chunk::ON_DISK);
 	}
-	/*	
-	void MultiFileCache::oldDownloadStatusChanged(TorrentFile* tf, bool download)
-	{
-		bool dnd = !download;
-		CacheFile* fd = files.find(tf->getIndex());
-		
-		QString dnd_dir = tmpdir + "dnd" + bt::DirSeparator();
-		// if it is dnd and it is already in the dnd tree do nothing
-		if (dnd && bt::Exists(dnd_dir + tf->getPath()))
-			return;
-		
-		// if it is !dnd and it is already in the output_dir tree do nothing
-		if (!dnd && bt::Exists(output_dir + tf->getPath()))
-			return;
-		if (fd)
-			fd->close(true);
-		
-		try
-		{
-			// now move it from output_dir tree to dnd tree or vica versa
-			// delete the symlink
-			bt::Delete(cache_dir + tf->getPath());
-			if (dnd)
-			{
-				bt::Move(output_dir + tf->getPath(),dnd_dir + tf->getPath());
-				bt::SymLink(dnd_dir + tf->getPath(),cache_dir + tf->getPath());
-			}
-			else
-			{
-				bt::Move(dnd_dir + tf->getPath(),output_dir + tf->getPath());
-				bt::SymLink(output_dir + tf->getPath(),cache_dir + tf->getPath());
-			}
-			
-		}
-		catch (bt::Error & e)
-		{
-			Out() << e.toString() << endl;
-		}
-		
-		if (fd)
-			fd->open(cache_dir + tf->getPath(),tf->getSize());
-	}
-	*/
+	
 	void MultiFileCache::downloadStatusChanged(TorrentFile* tf, bool download)
 	{	
 		bool dnd = !download;
@@ -573,7 +565,19 @@ namespace bt
 		// truncate it
 		try
 		{
-			bt::TruncateFile(output_file,tf->getSize());
+			bool res = false;
+			
+			#ifdef HAVE_XFS_XFS_H
+				if( (! res) && (Settings::fullDiskPreallocMethod() == 1) )
+				{
+					res = XfsPreallocate(output_file, tf->getSize());
+				}
+			#endif
+			
+			if(! res)
+			{
+				bt::TruncateFile(output_file,tf->getSize());
+			}
 		}
 		catch (bt::Error & e)
 		{
@@ -656,7 +660,7 @@ namespace bt
 			
 			QString p = cache_dir + tf.getPath();
 			QFileInfo fi(p);
-			// allways use symlink first, file might have been moved
+			// always use symlink first, file might have been moved
 			if (!fi.exists())
 			{
 				ret = true;
@@ -741,6 +745,42 @@ namespace bt
 			DeleteEmptyDirs(output_dir,fpath);
 		}
 	}
+	
+	Uint64 MultiFileCache::diskUsage()
+	{
+		Uint64 sum = 0;
+
+		for (Uint32 i = 0;i < tor.getNumFiles();i++)
+		{
+			TorrentFile & tf = tor.getFile(i);
+			if (tf.doNotDownload())
+				continue;
+
+			try
+			{
+				CacheFile* cf = files.find(i);
+				if (cf)
+				{
+					sum += cf->diskUsage();
+				}
+				else
+				{
+					// doesn't exist yet, must be before open is called
+					// so create one and delete it right after
+					cf = new CacheFile();
+					cf->open(cache_dir + tf.getPath(),tf.getSize());
+					sum += cf->diskUsage();
+					delete cf;
+				}
+			}
+			catch (bt::Error & err) // make sure we catch any exceptions
+			{
+				Out(SYS_DIO|LOG_DEBUG) << "Error: " << err.toString() << endl;
+			}
+		}
+
+		return sum;
+	}
 
 	///////////////////////////////
 
@@ -751,13 +791,7 @@ namespace bt
 	
 	Uint64 FileOffset(Uint32 cindex,const TorrentFile & f,Uint64 chunk_size)
 	{
-		Uint64 off = 0;
-		if (cindex - f.getFirstChunk() > 0)
-			off = (cindex - f.getFirstChunk() - 1) * chunk_size;
-		if (cindex > 0)
-			off += (chunk_size - f.getFirstChunkOffset());
-		return off;
+		return f.fileOffset(cindex,chunk_size);
 	}
 	
 }
-
