@@ -18,7 +18,9 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.             *
  ***************************************************************************/
 #include <util/log.h>
+#include <util/file.h>
 #include <util/error.h>
+#include <net/address.h>
 #include "peermanager.h"
 #include "peer.h"
 #include "bnode.h"
@@ -36,6 +38,9 @@
 #include "ipblocklist.h"
 #include "chunkcounter.h"
 #include "authenticationmonitor.h"
+#include <qdatetime.h>
+
+using namespace kt;
 
 namespace bt
 {
@@ -46,11 +51,12 @@ namespace bt
 	PeerManager::PeerManager(Torrent & tor)
 		: tor(tor),available_chunks(tor.getNumChunks())
 	{
-		num_seeders = num_leechers = num_pending = 0;
 		killed.setAutoDelete(true);
 		started = false;
 		
 		cnt = new ChunkCounter(tor.getNumChunks());
+		num_pending = 0;
+		pex_on = !tor.isPrivate();
 	}
 
 
@@ -70,7 +76,7 @@ namespace bt
 
 	void PeerManager::update()
 	{
-		if (!started || Globals::instance().inCriticalOperationMode())
+		if (!started)
 			return;
 
 		// update the speed of each peer,
@@ -92,7 +98,7 @@ namespace bt
 			}
 			else
 			{
-				p->update();
+				p->update(this);
 				i++;
 			}
 		}
@@ -104,7 +110,7 @@ namespace bt
 	void PeerManager::killChokedPeers(Uint32 older_then)
 	{
 		Out() << "Getting rid of peers which have been choked for a long time" << endl;
-		Uint32 now = bt::GetCurrentTime();
+		TimeStamp now = bt::GetCurrentTime();
 		QPtrList<Peer>::iterator i = peer_list.begin();
 		Uint32 num_killed = 0;
 		while (i != peer_list.end() && num_killed < 20)
@@ -127,15 +133,26 @@ namespace bt
 	
 	void PeerManager::setMaxTotalConnections(Uint32 max)
 	{
+		Uint32 sys_max = bt::MaxOpenFiles() - 50; // leave about 50 free for regular files
 		max_total_connections = max;
+		if (max == 0 || max_total_connections > sys_max)
+			max_total_connections = sys_max;
 	}
 	
 	void PeerManager::addPotentialPeer(const PotentialPeer & pp)
 	{
-//		Out() << "Addding " << pp.ip << ":" << pp.port << endl;
-		// lets not add to many potential peer, we can only connect to so many peers
-		if (potential_peers.count() < 100)
-			potential_peers.append(pp);
+		if (potential_peers.size() > 150)
+			return;
+		
+		// avoid duplicates in the potential_peers map
+		std::pair<PPItr,PPItr> r = potential_peers.equal_range(pp.ip);
+		for (PPItr i = r.first;i != r.second;i++)
+		{
+			if (i->second.port == pp.port) // port and IP are the same so return
+				return;
+		}
+		
+		potential_peers.insert(std::make_pair(pp.ip,pp));
 	}
 
 	void PeerManager::killSeeders()
@@ -149,6 +166,18 @@ namespace bt
 			i++;
 		}
 	}
+	
+	void PeerManager::killUninterested()
+	{
+		QPtrList<Peer>::iterator i = peer_list.begin();
+		while (i != peer_list.end())
+		{
+			Peer* p = *i;
+			if ( !p->isInterested() && (p->getConnectTime().secsTo(QTime::currentTime()) > 30) )
+				p->kill();
+			i++;
+		}
+	}
 
 	void PeerManager::onHave(Peer*,Uint32 index)
 	{
@@ -177,20 +206,16 @@ namespace bt
 		
 		if (!started || local_not_ok || global_not_ok)
 		{
-			delete sock;
-			return;
+			// get rid of bad peer and replace it by another one
+			if (!killBadPeer())
+			{
+				// we failed to find a bad peer, so just delete this one
+				delete sock;
+				return;
+			}
 		}
 
-		Peer* peer = new Peer(sock,peer_id,tor.getNumChunks(),tor.getChunkSize(),support);
-		connect(peer,SIGNAL(haveChunk(Peer*, Uint32 )),this,SLOT(onHave(Peer*, Uint32 )));
-		connect(peer,SIGNAL(bitSetRecieved(const BitSet& )),
-				this,SLOT(onBitSetRecieved(const BitSet& )));
-		connect(peer,SIGNAL(rerunChoker()),this,SLOT(onRerunChoker()));
-		
-		peer_list.append(peer);
-		peer_map.insert(peer->getID(),peer);
-		total_connections++;
-		newPeer(peer);
+		createPeer(sock,peer_id,support,false);
 	}
 	
 	void PeerManager::peerAuthenticated(Authenticate* auth,bool ok)
@@ -211,6 +236,9 @@ namespace bt
 				QString ip = a->getIP();
 				Uint16 port = a->getPort();
 				Authenticate* st = new Authenticate(ip,port,tor.getInfoHash(),tor.getPeerID(),this);
+				if (auth->isLocal())
+					st->setLocal(true);
+				
 				connect(this,SIGNAL(stopped()),st,SLOT(onPeerManagerDestroyed()));
 				AuthenticationMonitor::instance().add(st);
 				num_pending++;
@@ -224,23 +252,24 @@ namespace bt
 			return;
 		}
 			
-		Uint32 flags = 0;
-		if (auth->supportsDHT())
-			flags |= bt::DHT_SUPPORT;
-		if (auth->supportsFastExtensions())
-			flags |= bt::FAST_EXT_SUPPORT;
+		createPeer(auth->takeSocket(),auth->getPeerID(),auth->supportedExtensions(),auth->isLocal());
+	}
+	
+	void PeerManager::createPeer(mse::StreamSocket* sock,const PeerID & peer_id,Uint32 support,bool local)
+	{
+		Peer* peer = new Peer(sock,peer_id,tor.getNumChunks(),tor.getChunkSize(),support,local);
 		
-		Peer* peer = new Peer(auth->takeSocket(),auth->getPeerID(),tor.getNumChunks(),tor.getChunkSize(),flags);
 		connect(peer,SIGNAL(haveChunk(Peer*, Uint32 )),this,SLOT(onHave(Peer*, Uint32 )));
 		connect(peer,SIGNAL(bitSetRecieved(const BitSet& )),
 				this,SLOT(onBitSetRecieved(const BitSet& )));
 		connect(peer,SIGNAL(rerunChoker()),this,SLOT(onRerunChoker()));
+		connect(peer,SIGNAL(pex( const QByteArray& )),this,SLOT(pex( const QByteArray& )));
 		
 		peer_list.append(peer);
 		peer_map.insert(peer->getID(),peer);
 		total_connections++;
-		//	Out() << "New peer connected !" << endl;
 		newPeer(peer);
+		peer->setPexEnabled(pex_on);
 	}
 		
 	bool PeerManager::connectedTo(const PeerID & peer_id)
@@ -259,9 +288,22 @@ namespace bt
 		return false;
 	}
 	
+	bool PeerManager::connectedTo(const QString & ip,Uint16 port) const
+	{
+		PtrMap<Uint32,Peer>::const_iterator i = peer_map.begin();
+		while (i != peer_map.end())
+		{
+			const Peer* p = i->second;
+			if (p->getPort() == port && p->getStats().ip_address == ip)
+				return true;
+			i++;
+		}
+		return false;
+	}
+	
 	void PeerManager::connectToPeers()
 	{
-		if (potential_peers.count() == 0)
+		if (potential_peers.size() == 0)
 			return;
 		
 		if (peer_list.count() + num_pending >= max_connections && max_connections > 0)
@@ -277,12 +319,12 @@ namespace bt
 		if (max_connections > 0)
 		{
 			Uint32 available = max_connections - (peer_list.count() + num_pending);
-			num = available >= potential_peers.count() ? 
-					potential_peers.count() : available;
+			num = available >= potential_peers.size() ? 
+					potential_peers.size() : available;
 		}
 		else
 		{
-			num = potential_peers.count();
+			num = potential_peers.size();
 		}
 		
 		if (num + total_connections >= max_total_connections && max_total_connections > 0)
@@ -293,30 +335,31 @@ namespace bt
 			if (num_pending > MAX_SIMULTANIOUS_AUTHS)
 				return;
 			
-			PotentialPeer pp = potential_peers.front();
-			potential_peers.pop_front();
+			PPItr itr = potential_peers.begin();
 			
-			if (connectedTo(pp.id))
-				continue;
-
 			IPBlocklist& ipfilter = IPBlocklist::instance();
 			
-			if (ipfilter.isBlocked(pp.ip))
-				continue;
-			
-
-		//	Out() << "EncryptedAuthenticate : " << pp.ip << ":" << pp.port << endl;
-			Authenticate* auth = 0;
-			
-			if (Globals::instance().getServer().isEncryptionEnabled())
-				auth = new mse::EncryptedAuthenticate(pp.ip,pp.port,
-			tor.getInfoHash(),tor.getPeerID(),this);
-			else
-				auth = new Authenticate(pp.ip,pp.port,tor.getInfoHash(),tor.getPeerID(),this);
-			connect(this,SIGNAL(stopped()),auth,SLOT(onPeerManagerDestroyed()));
-			AuthenticationMonitor::instance().add(auth);
-			num_pending++;
-			total_connections++;
+			if (!ipfilter.isBlocked(itr->first) && !connectedTo(itr->first,itr->second.port))
+			{
+			//	Out() << "EncryptedAuthenticate : " << pp.ip << ":" << pp.port << endl;
+				Authenticate* auth = 0;
+				const PotentialPeer & pp = itr->second;
+				
+				if (Globals::instance().getServer().isEncryptionEnabled())
+					auth = new mse::EncryptedAuthenticate(pp.ip,pp.port,tor.getInfoHash(),tor.getPeerID(),this);
+				else
+					auth = new Authenticate(pp.ip,pp.port,tor.getInfoHash(),tor.getPeerID(),this);
+				
+				if (pp.local)
+					auth->setLocal(true);
+				
+				connect(this,SIGNAL(stopped()),auth,SLOT(onPeerManagerDestroyed()));
+				
+				AuthenticationMonitor::instance().add(auth);
+				num_pending++;
+				total_connections++;
+			}
+			potential_peers.erase(itr);
 		}
 	}
 	
@@ -332,7 +375,7 @@ namespace bt
 	void PeerManager::closeAllConnections()
 	{
 		killed.clear();
-		
+	
 		if (peer_list.count() <= total_connections)
 			total_connections -= peer_list.count();
 		else
@@ -344,6 +387,106 @@ namespace bt
 		peer_list.setAutoDelete(false);
 	}
 	
+	// pick a random magic number
+	const Uint32 PEER_LIST_HDR_MAGIC = 0xEF12AB34;
+	
+	struct PeerListHeader
+	{
+		Uint32 magic;
+		Uint32 num_peers;
+		Uint32 ip_version; // 4 or 6, 6 is for future purposes only (when we support IPv6)
+	};
+	
+	struct PeerListEntry
+	{
+		Uint32 ip;
+		Uint16 port;
+	};
+	
+	void PeerManager::savePeerList(const QString & file)
+	{
+		bt::File fptr;
+		if (!fptr.open(file,"wb"))
+			return;
+		
+		try
+		{
+			PeerListHeader hdr;
+			hdr.magic = PEER_LIST_HDR_MAGIC;
+			// we will save both the active and potential peers
+			hdr.num_peers = peer_list.count() + potential_peers.size();
+			hdr.ip_version = 4;
+			
+			fptr.write(&hdr,sizeof(PeerListHeader));
+			
+			Out(SYS_GEN|LOG_DEBUG) << "Saving list of peers to " << file << endl;
+			// first the active peers
+			for (QPtrList<Peer>::iterator itr = peer_list.begin(); itr != peer_list.end();itr++)
+			{
+				Peer* p = *itr;
+				PeerListEntry e;
+				net::Address addr = p->getAddress();
+				e.ip = addr.ip();
+				e.port = addr.port();
+				fptr.write(&e,sizeof(PeerListEntry));
+			}
+			
+			// now the potential_peers
+			PPItr i = potential_peers.begin();
+			while (i != potential_peers.end())
+			{
+				net::Address addr(i->first,i->second.port);
+				PeerListEntry e;
+				e.ip = addr.ip();
+				e.port = addr.port();
+				fptr.write(&e,sizeof(PeerListEntry));
+				i++;
+			}
+		}
+		catch (bt::Error & err)
+		{
+			Out(SYS_GEN|LOG_DEBUG) << "Error happened during saving of peer list : " << err.toString() << endl;
+		}
+	}
+	
+	void PeerManager::loadPeerList(const QString & file)
+	{
+		bt::File fptr;
+		if (!fptr.open(file,"rb"))
+			return;
+		
+		try
+		{
+			PeerListHeader hdr;
+			fptr.read(&hdr,sizeof(PeerListHeader));
+			if (hdr.magic != PEER_LIST_HDR_MAGIC || hdr.ip_version != 4)
+				throw Error("Peer list file corrupted");
+			
+			Out(SYS_GEN|LOG_DEBUG) << "Loading list of peers from " << file << " (num_peers =  " << hdr.num_peers << ")" << endl;
+			
+			for (Uint32 i = 0;i < hdr.num_peers && !fptr.eof();i++)
+			{
+				PeerListEntry e;
+				fptr.read(&e,sizeof(PeerListEntry));
+				PotentialPeer pp;
+				
+				// convert IP address to string 
+				pp.ip = QString("%1.%2.%3.%4")
+						.arg((e.ip & 0xFF000000) >> 24)
+						.arg((e.ip & 0x00FF0000) >> 16)
+						.arg((e.ip & 0x0000FF00) >> 8)
+						.arg( e.ip & 0x000000FF);
+				pp.port = e.port;
+				addPotentialPeer(pp);
+			}
+			
+		}
+		catch (bt::Error & err)
+		{
+			Out(SYS_GEN|LOG_DEBUG) << "Error happened during saving of peer list : " << err.toString() << endl;
+		}
+	}
+	
 	void PeerManager::start()
 	{
 		started = true;
@@ -382,6 +525,80 @@ namespace bt
 		}
 	}
 	
+	void PeerManager::peerSourceReady(kt::PeerSource* ps)
+	{
+		PotentialPeer pp;
+		while (ps->takePotentialPeer(pp))
+			addPotentialPeer(pp);
+	}
 	
+	bool PeerManager::killBadPeer()
+	{
+		for (PtrMap<Uint32,Peer>::iterator i = peer_map.begin();i != peer_map.end();i++)
+		{
+			Peer* p = i->second;
+			if (p->getStats().aca_score <= -5.0 && p->getStats().aca_score > -50.0)
+			{
+				Out(SYS_GEN|LOG_DEBUG) << "Killing bad peer, to make room for other peers" << endl;
+				p->kill();
+				return true;
+			}
+		}
+		return false;
+	}
+	
+	void PeerManager::pex(const QByteArray & arr)
+	{
+		if (!pex_on)
+			return;
+		
+		Out(SYS_CON|LOG_NOTICE) << "PEX: found " << (arr.size() / 6) << " peers"  << endl;
+		for (Uint32 i = 0;i+6 <= arr.size();i+=6)
+		{
+			Uint8 tmp[6];
+			memcpy(tmp,arr.data() + i,6);
+			PotentialPeer pp;
+			pp.port = ReadUint16(tmp,4);
+			Uint32 ip = ReadUint32(tmp,0);
+			pp.ip = QString("%1.%2.%3.%4")
+						.arg((ip & 0xFF000000) >> 24)
+						.arg((ip & 0x00FF0000) >> 16)
+						.arg((ip & 0x0000FF00) >> 8)
+						.arg( ip & 0x000000FF);
+			pp.local = false;
+			
+			addPotentialPeer(pp);
+		}
+	}
+	
+	
+	void PeerManager::setPexEnabled(bool on)
+	{
+		if (on && tor.isPrivate())
+			return;
+		
+		if (pex_on == on)
+			return;
+		
+		QPtrList<Peer>::iterator i = peer_list.begin();
+		while (i != peer_list.end())
+		{
+			Peer* p = *i;
+			if (!p->isKilled())
+				p->setPexEnabled(on);
+			i++;
+		}
+		pex_on = on;
+	}
+	
+	void PeerManager::setGroupIDs(Uint32 up,Uint32 down)
+	{
+		for (PtrMap<Uint32,Peer>::iterator i = peer_map.begin();i != peer_map.end();i++)
+		{
+			Peer* p = i->second;
+			p->setGroupIDs(up,down);
+		}
+	}
 }
+
 #include "peermanager.moc"
