@@ -48,11 +48,13 @@ namespace bt
 		virtual void create();
 		virtual void close();
 		virtual void open();
-		virtual void changeDataDir(const QString & ndir);
+		virtual void changeTmpDir(const QString & ndir);
+		virtual void changeOutputPath(const QString& outputpath);
 		virtual QString getOutputPath() const {return output_file;}
 		virtual void preallocateDiskSpace(PreallocationThread* prealloc);
 		virtual bool hasMissingFiles(QStringList & sl);
 		virtual void deleteDataFiles();
+		virtual Uint64 diskUsage();
 	};
 
 }
