@@ -1,6 +1,6 @@
 /***************************************************************************
- *   Copyright (C) 2005 by Joris Guisson                                   *
- *   joris.guisson@gmail.com                                               *
+ *   Copyright (C) 2005 by Joris Guisson <joris.guisson@gmail.com>         *
+ *   Copyright (C) 2007 by Modestas Vainius <modestas@vainius.eu>          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -15,7 +15,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.          *
  ***************************************************************************/
 #include <klocale.h>
 #include <kglobal.h>
@@ -29,8 +29,15 @@
 #include <interfaces/peerinterface.h>
 #include <interfaces/functions.h>
 #include <torrent/ipblocklist.h>
-#include "peerview.h"
+#include "config.h"
+
+#ifdef USE_SYSTEM_GEOIP
+#include <GeoIP.h>
+#else
 #include "GeoIP.h"
+#endif
+#include "peerview.h"
+#include "flagdb.h"
 
 using namespace bt;
 using namespace kt;
@@ -44,19 +51,35 @@ namespace kt
 	static QPixmap yes_pix;
 	static QPixmap no_pix;
 	static QPixmap lock_pix;
+	static FlagDB flagDB(22, 18);
 	static bool yes_no_pix_loaded = false;
 	static bool geoip_db_exists = true;
-	
+	static QString geoip_data_file;
 		
 	PeerViewItem::PeerViewItem(PeerView* pv,kt::PeerInterface* peer) : KListViewItem(pv),peer(peer)
 	{	
 		if (!yes_no_pix_loaded)
 		{
 			KIconLoader* iload = KGlobal::iconLoader();
+			/* Prefer builtin flag images to the ones provided by KDE */
+			flagDB.addFlagSource("data",  QString("ktorrent/geoip/%1.png"));
+			flagDB.addFlagSource("locale", QString("l10n/%1/flag.png"));
 			yes_pix = iload->loadIcon("button_ok",KIcon::Small);
 			no_pix = iload->loadIcon("button_cancel",KIcon::Small);
 			lock_pix = iload->loadIcon("ktencrypted",KIcon::Small);
-			geoip_db_exists = !locate("data", "ktorrent/geoip/geoip.dat").isNull();
+#ifdef USE_SYSTEM_GEOIP
+			geo_ip = GeoIP_open_type(GEOIP_COUNTRY_EDITION, GEOIP_STANDARD);
+			geoip_db_exists = (geo_ip != NULL);
+#else
+			geoip_db_exists = !locate("data", "ktorrent/geoip/geoip.dat").isNull(); 
+			if(geoip_db_exists) {
+				geoip_data_file = "ktorrent/geoip/geoip.dat";
+			} else {
+				geoip_db_exists = !locate("data", "ktorrent/geoip/GeoIP.dat").isNull();
+				if (geoip_db_exists)
+					geoip_data_file = "ktorrent/geoip/GeoIP.dat";
+			}
+#endif
 			yes_no_pix_loaded = true;
 		}
 		
@@ -67,12 +90,16 @@ namespace kt
 		int country_id = 0;
 
 		const PeerInterface::Stats & s = peer->getStats();
-		hostname = s.ip_addresss.ascii();
+		hostname = s.ip_address.ascii();
 
 		// open GeoIP if necessary
-		if (!geo_ip && geoip_db_exists)
-			geo_ip = GeoIP_open(locate("data", "ktorrent/geoip/geoip.dat").ascii(),0);
-		
+		if (!geo_ip && geoip_db_exists) {
+#ifdef USE_SYSTEM_GEOIP
+			geo_ip = GeoIP_open_type(GEOIP_COUNTRY_EDITION, GEOIP_STANDARD);
+#else
+			geo_ip = GeoIP_open(locate("data", geoip_data_file).ascii(),0);
+#endif
+		}
 		if (geo_ip)
 		{
 			country_id = GeoIP_id_by_name(geo_ip, hostname);
@@ -86,13 +113,12 @@ namespace kt
 			setText(1,"N/A");
 		}
 		
-		setText(0,s.ip_addresss);
+		setText(0,s.ip_address);
 		setText(2,s.client);
 		
 		if (country_code)
 		{
-			QPixmap pix(locate("data", 	QString("ktorrent/geoip/%1.png").arg(QString(country_code)).lower()));
-			setPixmap(1, pix);
+			setPixmap(1, flagDB.getFlag(country_code));
 		}
 		
 		if (s.encrypted)
@@ -118,9 +144,15 @@ namespace kt
 	{
 		KLocale* loc = KGlobal::locale();
 		const PeerInterface::Stats & s = peer->getStats();
-		
-		setText(3,KBytesPerSecToString(s.download_rate / 1024.0));
-		setText(4,KBytesPerSecToString(s.upload_rate / 1024.0));
+
+		if (s.download_rate >= 103) // lowest "visible" speed, all below will be 0,0 Kb/s
+			setText(3,KBytesPerSecToString(s.download_rate / 1024.0));
+		else
+			setText(3, "");
+		if (s.upload_rate >= 103) // lowest "visible" speed, all below will be 0,0 Kb/s
+			setText(4,KBytesPerSecToString(s.upload_rate / 1024.0));
+		else
+			setText(4, "");
 		//setPixmap(5,!s.choked ? yes_pix : no_pix);
 		setText(5,s.choked ? i18n("Yes") : i18n("No"));
 		//setPixmap(6,!s.snubbed ? yes_pix : no_pix);
@@ -129,10 +161,9 @@ namespace kt
 		setPixmap(8,s.dht_support ? yes_pix : no_pix);
 		setText(9,loc->formatNumber(s.aca_score,2));
 		setPixmap(10,s.has_upload_slot ? yes_pix : QPixmap());
-#undef SHOW_REQUESTS
-#ifdef SHOW_REQUESTS
 		setText(11,QString("%1 / %2").arg(s.num_down_requests).arg(s.num_up_requests));
-#endif
+		setText(12, BytesToString(s.bytes_downloaded));
+		setText(13, BytesToString(s.bytes_uploaded));
 	}
 	
 	int PeerViewItem::compare(QListViewItem * i,int col,bool) const
@@ -143,7 +174,7 @@ namespace kt
 		const PeerInterface::Stats & os = op->getStats();
 		switch (col)
 		{
-			case 0: return QString::compare(s.ip_addresss,os.ip_addresss);
+			case 0: return QString::compare(s.ip_address,os.ip_address);
 			case 1:	
 			{
 				int ret = QString::compare(m_country, pvi->m_country);
@@ -158,6 +189,10 @@ namespace kt
 			case 8: return CompareVal(s.dht_support,os.dht_support);
 			case 9: return CompareVal(s.aca_score,os.aca_score);
 			case 10: return CompareVal(s.has_upload_slot,os.has_upload_slot);
+			case 11: return CompareVal(s.num_down_requests+s.num_up_requests, os.num_down_requests+os.num_up_requests);
+			case 12: return CompareVal(s.bytes_downloaded, os.bytes_downloaded);
+			case 13: return CompareVal(s.bytes_uploaded, os.bytes_uploaded);
+			
 		}
 		return 0;
 	}
@@ -176,17 +211,38 @@ namespace kt
 		addColumn(i18n("DHT"));
 		addColumn(i18n("Score"));
 		addColumn(i18n("Upload Slot"));
-#ifdef SHOW_REQUESTS
 		addColumn(i18n("Requests"));
-#endif
+		addColumn(i18n("Downloaded"));
+		addColumn(i18n("Uploaded"));
+		
+		setAllColumnsShowFocus(true);
+		setShowSortIndicator(true);
+		
+		setColumnAlignment(3,Qt::AlignRight);
+		setColumnAlignment(4,Qt::AlignRight);
+		setColumnAlignment(5,Qt::AlignCenter);
+		setColumnAlignment(6,Qt::AlignCenter);
+		setColumnAlignment(7,Qt::AlignRight);
+		setColumnAlignment(8,Qt::AlignCenter);
+		setColumnAlignment(9,Qt::AlignRight);
+		setColumnAlignment(10,Qt::AlignCenter);
+		setColumnAlignment(11,Qt::AlignCenter);
+		setColumnAlignment(12,Qt::AlignRight);
+		setColumnAlignment(13,Qt::AlignRight);
+		
+		for (Uint32 i = 0;i < (Uint32)columns();i++)
+			setColumnWidthMode(i,QListView::Manual);
+			
 		setShowSortIndicator(true);
 		
 		menu = new KPopupMenu(this);
-		ban_id = menu->insertItem(KGlobal::iconLoader()->loadIcon("filter",KIcon::NoGroup), i18n("to ban", "Ban Peer"));
+		kick_id = menu->insertItem(KGlobal::iconLoader()->loadIcon("delete_user", KIcon::NoGroup), i18n("to kick", "Kick peer"));
+		ban_id = menu->insertItem(KGlobal::iconLoader()->loadIcon("filter",KIcon::NoGroup), i18n("to ban", "Ban peer"));
 		
 		connect(this,SIGNAL(contextMenu(KListView*, QListViewItem*, const QPoint& )),
 				this,SLOT(showContextMenu(KListView*, QListViewItem*, const QPoint& )));
 		connect(menu, SIGNAL ( activated ( int ) ), this, SLOT ( contextItem ( int ) ) );
+		setFrameShape(QFrame::NoFrame);
 	}
 	
 	
@@ -223,7 +279,7 @@ namespace kt
 		
 		IPBlocklist& filter = IPBlocklist::instance();
 		const PeerInterface::Stats & s = peer->getStats();
-		KNetwork::KIpAddress ip(s.ip_addresss);
+		KNetwork::KIpAddress ip(s.ip_address);
 		QString ips = ip.toString();
 		/**
 		 * @TODO Clean this up.
@@ -238,6 +294,14 @@ namespace kt
 		peer->kill();
 	}
 	
+	void PeerView::kickPeer(kt::PeerInterface* peer)
+	{
+		if(!peer)
+			return;
+		
+		peer->kill();
+	}
+	
 	void PeerView::update()
 	{
 		QMap<kt::PeerInterface*,PeerViewItem*>::iterator i = items.begin();
@@ -265,14 +329,18 @@ namespace kt
 		if (curr)
 		{
 			menu->setItemEnabled(ban_id, true);
+			menu->setItemEnabled(kick_id, true);
 			menu->popup(p);
 		}
 	}
 	
-	void PeerView::contextItem( int id )
+	void PeerView::contextItem(int id)
 	{
 		if (id == ban_id && curr)
 			banPeer(curr->getPeer());
+		
+		if (id == kick_id && curr)
+			kickPeer(curr->getPeer());
 	}
 }
 	
