@@ -29,7 +29,7 @@
 #include "logprefpage.h"
 #include "logflags.h"
 
-#define NAME "logviewerplugin"
+#define NAME "Log Viewer"
 #define AUTHOR "Joris Guisson"
 #define EMAIL "joris.guisson@gmail.com"
 
@@ -41,7 +41,8 @@ namespace kt
 {
 
 	LogViewerPlugin::LogViewerPlugin(QObject* parent, const char* qt_name, const QStringList& args)
-	: Plugin(parent, qt_name, args, NAME, AUTHOR, EMAIL,i18n("Shows logging output in a widget"))
+	: Plugin(parent, qt_name, args, NAME, AUTHOR, EMAIL,
+			 i18n("Displays ktorrent logging output"),"log")
 	{
 		lv = 0;
 	}
@@ -54,7 +55,7 @@ namespace kt
 	void LogViewerPlugin::load()
 	{
 		lv = new LogViewer();
-		this->getGUI()->addWidgetBelowView(lv,"log",i18n("Log Viewer"));
+		this->getGUI()->addToolWidget(lv,"log",i18n("Log Viewer"),GUIInterface::DOCK_BOTTOM);
 		bt::Log & lg = Globals::instance().getLog(0);
 		lg.addMonitor(lv);
 		pref = new LogPrefPage();
@@ -63,7 +64,7 @@ namespace kt
 
 	void LogViewerPlugin::unload()
 	{
-		this->getGUI()->removeWidgetBelowView(lv);
+		this->getGUI()->removeToolWidget(lv);
 		bt::Log & lg = Globals::instance().getLog(0);
 		lg.removeMonitor(lv);
 		delete lv;
