@@ -28,7 +28,7 @@
 #include "partfileimportplugin.h"
 #include "importdialog.h"
 
-#define NAME "partfileimportplugin"
+#define NAME "Import"
 #define AUTHOR "Joris Guisson"
 #define EMAIL "joris.guisson@gmail.com"
 
@@ -40,7 +40,7 @@ namespace kt
 {
 
 	PartFileImportPlugin::PartFileImportPlugin(QObject* parent, const char* name, const QStringList& args)
-	: Plugin(parent, name, args,NAME,AUTHOR,EMAIL,i18n("Imports partially or fully downloaded torrents from other clients"))
+	: Plugin(parent, name, args,NAME,AUTHOR,EMAIL,i18n("Imports partially or fully downloaded torrents from other clients"),"ktplugins")
 	{
 		setXMLFile("ktpartfileimportpluginui.rc");
 		import_action = 0;
