@@ -23,13 +23,19 @@
 #include <kiconloader.h>
 #include <kstdaction.h>
 #include <kpopupmenu.h>
+#include <kapplication.h>
+#include <kstandarddirs.h>
+#include <krun.h>
 #include <interfaces/guiinterface.h>
 #include "searchplugin.h"
 #include "searchwidget.h"
 #include "searchprefpage.h"
+#include "searchtab.h"
+#include "searchpluginsettings.h"
+#include "searchenginelist.h"
 
 
-#define NAME "searchplugin"
+#define NAME "Search"
 #define AUTHOR "Joris Guisson"
 #define EMAIL "joris.guisson@gmail.com"
 
@@ -41,11 +47,12 @@ namespace kt
 {
 
 	SearchPlugin::SearchPlugin(QObject* parent, const char* name, const QStringList& args)
-	: Plugin(parent, name, args,NAME,AUTHOR,EMAIL,i18n("Search for torrents on several popular torrent search engines"))
+	: Plugin(parent, name, args,NAME,AUTHOR,EMAIL,
+			 i18n("Search for torrents on several popular torrent search engines"),"viewmag")
 	{
 		// setXMLFile("ktsearchpluginui.rc");
-		search = 0;
 		pref = 0;
+		tab = 0;
 	}
 
 
@@ -55,36 +62,96 @@ namespace kt
 
 	void SearchPlugin::load()
 	{
-		KIconLoader* iload = KGlobal::iconLoader();
-		search = new SearchWidget(this);
-		getGUI()->addTabPage(
-				search,iload->loadIconSet("viewmag", KIcon::Small),
-				i18n("Search"));
-		
-		
+		engines.load(KGlobal::dirs()->saveLocation("data","ktorrent") + "search_engines");
+		KToolBar* tb = getGUI()->addToolBar("search");
+		tab = new SearchTab(tb);
+		connect(tab,SIGNAL(search( const QString&, int, bool )),
+				this,SLOT(search( const QString&, int, bool )));
+		 
 		pref = new SearchPrefPage(this);
 		getGUI()->addPrefPage(pref);
-
-		KAction* copy_act = KStdAction::copy(search,SLOT(copy()),actionCollection());
-		copy_act->plug(search->rightClickMenu(),0);
+		pref->updateData();
+		tab->updateSearchEngines(engines);
 	}
 
 	void SearchPlugin::unload()
 	{
-		search->onShutDown();
-		getGUI()->removeTabPage(search);
+		tab->saveSettings();
+		SearchWidget* s = 0;
+		while ((s = searches.first()) != 0)
+		{
+			getGUI()->removeTabPage(s);
+			searches.removeFirst();
+			delete s;
+		}
+		getGUI()->removeToolBar(tab->getToolBar());
 		getGUI()->removePrefPage(pref);
-		delete search;
-		search = 0;
 		delete pref;
 		pref = 0;
+		delete tab;
+		tab = 0;
+	}
+	
+	void SearchPlugin::search(const QString & text,int engine,bool external)
+	{	
+		if(external)
+		{
+			const SearchEngineList& sl = getSearchEngineList();
+		
+			if (engine < 0 || engine >= sl.getNumEngines())
+				engine = 0;
+		
+			QString s_url = sl.getSearchURL(engine).prettyURL();
+			s_url.replace("FOOBAR", KURL::encode_string(text), true);
+			KURL url = KURL::fromPathOrURL(s_url);
+			
+			if(SearchPluginSettings::useDefaultBrowser())
+				kapp->invokeBrowser(url.url());
+			else
+				KRun::runCommand(QString("%1 \"%2\"").arg(SearchPluginSettings::customBrowser()).arg(url.url()), SearchPluginSettings::customBrowser(), "viewmag" );
+			
+			return;
+		}
+		
+		KIconLoader* iload = KGlobal::iconLoader();
+		
+		SearchWidget* search = new SearchWidget(this);
+		getGUI()->addTabPage(search,iload->loadIconSet("viewmag", KIcon::Small),text,this);
+		
+		KAction* copy_act = KStdAction::copy(search,SLOT(copy()),actionCollection());
+		copy_act->plug(search->rightClickMenu(),0);
+		searches.append(search);
+		
+		search->updateSearchEngines(engines);
+		search->search(text,engine);
 	}
 	
 	void SearchPlugin::preferencesUpdated()
 	{
-		if(search)
-			search->loadSearchEngines();
+		engines.load(KGlobal::dirs()->saveLocation("data","ktorrent") + "search_engines");
+		if (tab)
+			tab->updateSearchEngines(engines);
+		
+		for (QPtrList<SearchWidget>::iterator i = searches.begin(); i != searches.end();i++)
+		{
+			SearchWidget* w = *i;
+			w->updateSearchEngines(engines);
+		}
+	}
+	
+	void SearchPlugin::tabCloseRequest(kt::GUIInterface* gui,QWidget* tab)
+	{
+		if (searches.contains((SearchWidget*)tab))
+		{
+			searches.remove((SearchWidget*)tab);
+			gui->removeTabPage(tab);
+			tab->deleteLater();
+		}
 	}
 
+	bool SearchPlugin::versionCheck(const QString & version) const
+	{
+		return version == KT_VERSION_MACRO;
+	}
 }
 #include "searchplugin.moc"
