@@ -19,6 +19,7 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.             *
  ***************************************************************************/
 #include <kurl.h>
+#include <qtooltip.h>
 #include <qfile.h>
 #include <klocale.h>
 #include <kglobal.h>
@@ -30,10 +31,17 @@
 #include <klineedit.h>
 #include <kmessagebox.h>
 #include <kio/netaccess.h>
+#include <klineedit.h>
+
+#include <qlabel.h>
+#include <qcheckbox.h>
+#include <qradiobutton.h>
 
 #include <util/constants.h>
 #include "searchprefpage.h"
 #include "searchplugin.h"
+#include "searchenginelist.h"
+#include "searchpluginsettings.h"
 
 using namespace bt;
 
@@ -42,57 +50,50 @@ namespace kt
 	SearchPrefPageWidget::SearchPrefPageWidget(QWidget *parent) : SEPreferences(parent)
 	{
 		QString info = i18n("Use your web browser to search for the string %1"
-				" (capital letters) on the search engine you want to add. Then copy the URL in the addressbar after the search is finished, and paste it here.<br>Searching for %2"
-				" on Google for example, will result in http://www.google.com/search?q=FOOBAR&ie=UTF-8&oe=UTF-8. If you add this URL here, ktorrent can search using Google.").arg("FOOBAR").arg("FOOBAR");
-		m_infoLabel->setText(info);
-		loadSearchEngines();
+				" (capital letters) on the search engine you want to add. <br> Then copy the URL in the addressbar after the search is finished, and paste it here.<br><br>Searching for %2"
+				" on Google for example, will result in http://www.google.com/search?q=FOOBAR&ie=UTF-8&oe=UTF-8. <br> If you add this URL here, ktorrent can search using Google.").arg("FOOBAR").arg("FOOBAR");
+		QString info_short = i18n("Use your web browser to search for the string %1 (capital letters) on the search engine you want to add. Use the resulting URL below.").arg("FOOBAR");
+		m_infoLabel->setText(info_short);
+		QToolTip::add(m_infoLabel,info);
+		QToolTip::add(m_engine_name,info);
+		
 		connect(btnAdd, SIGNAL(clicked()), this, SLOT(addClicked()));
 		connect(btnRemove, SIGNAL(clicked()), this, SLOT(removeClicked()));
 		connect(btn_add_default, SIGNAL(clicked()), this, SLOT(addDefaultClicked()));
 		connect(btnRemoveAll, SIGNAL(clicked()), this, SLOT(removeAllClicked()));
+		
+		connect(useCustomBrowser, SIGNAL(toggled(bool)), this, SLOT(customToggled( bool )));
+		
+		useCustomBrowser->setChecked(SearchPluginSettings::useCustomBrowser());
+		useDefaultBrowser->setChecked(SearchPluginSettings::useDefaultBrowser());
+		customBrowser->setText(SearchPluginSettings::customBrowser());
+		
+		customBrowser->setEnabled(useCustomBrowser->isChecked());
+		openExternal->setChecked(SearchPluginSettings::openInExternal());
+	}
+	
+	void SearchPrefPageWidget::updateSearchEngines(const SearchEngineList & se)
+	{
+		m_engines->clear();
+		
+		for (Uint32 i = 0;i < se.getNumEngines();i++)
+		{
+			new QListViewItem(m_engines,se.getEngineName(i),se.getSearchURL(i).prettyURL());
+		}
 	}
  
 	bool SearchPrefPageWidget::apply()
 	{
 		saveSearchEngines();
+		
+		SearchPluginSettings::setUseCustomBrowser(useCustomBrowser->isChecked());
+		SearchPluginSettings::setUseDefaultBrowser(useDefaultBrowser->isChecked());
+		SearchPluginSettings::setCustomBrowser(customBrowser->text());
+		SearchPluginSettings::setOpenInExternal(openExternal->isChecked());
+		SearchPluginSettings::writeConfig();
 		return true;
 	}
  
-	void SearchPrefPageWidget::loadSearchEngines()
-	{
-		m_items.clear();
-		m_engines->clear();
-		
-		QFile fptr(KGlobal::dirs()->saveLocation("data","ktorrent") + "search_engines");
-     
-		if (!fptr.open(IO_ReadOnly))
-			return;
- 
-		QTextStream in(&fptr);
-		int id = 0;
-
-		while (!in.atEnd())
-		{
-			QString line = in.readLine();
-
-			if(line.startsWith("#") || line.startsWith(" ") || line.isEmpty() )
-				continue;
-
-			QStringList tokens = QStringList::split(" ", line);
-			QString name = tokens[0];
-			name = name.replace("%20"," ");
-			KURL url = KURL::fromPathOrURL(tokens[1]);
-			for(Uint32 i=2; i<tokens.count(); ++i)
-				url.addQueryItem(tokens[i].section("=",0,0), tokens[i].section("=", 1, 1));
-		
-			QListViewItem* se = new QListViewItem(m_engines, name, url.url());
-			m_items.append(se);
-			m_engines->insertItem(se);
-
-			++id;
-		}
-	}
- 
 	void SearchPrefPageWidget::saveSearchEngines()
 	{
 		QFile fptr(KGlobal::dirs()->saveLocation("data","ktorrent") + "search_engines");
@@ -102,96 +103,81 @@ namespace kt
 		out << "# PLEASE DO NOT MODIFY THIS FILE. Use KTorrent configuration dialog for adding new search engines." << ::endl;
 		out << "# SEARCH ENGINES list" << ::endl;
      
-		for(Uint32 i=0; i<m_items.count(); ++i)
+		QListViewItemIterator itr(m_engines);
+		while (itr.current())
 		{
-			QListViewItem* item = m_items.at(i);
+			QListViewItem* item = itr.current();
 			QString u = item->text(1);
-			QMap<QString,QString> args = KURL(u).queryItems();
-
-			// replace spaces by %20
 			QString name = item->text(0);
-			name = name.replace(" ","%20");
-	
-			out << name << " " << u.section("?",0,0) << " ";
-			
-			for (QMap<QString,QString>::iterator j = args.begin();j != args.end();j++)
-				out << j.key() << "=" << j.data() << " ";
-			out << endl;
+			out << name.replace(" ","%20") << " " << u.replace(" ","%20") <<  endl;
+			itr++;
 		}
 	}
  
 	void SearchPrefPageWidget::addClicked()
 	{
 		if ( m_engine_url->text().isEmpty() || m_engine_name->text().isEmpty() )
-			KMessageBox::error(this, i18n("You must enter the search engine's name and URL"), i18n("Error"));
+		{
+			KMessageBox::error(this, i18n("You must enter the search engine's name and URL"));
+		}
 		else if ( m_engine_url->text().contains("FOOBAR")  )
 		{
 			KURL url = KURL::fromPathOrURL(m_engine_url->text());
-			if ( !url.isValid() ) { KMessageBox::error(this, i18n("Malformed URL."), i18n("Error")); return; }
-			if (m_engines->findItem(m_engine_name->text(), 0)) { KMessageBox::error(this, i18n("A search engine with the same name already exists. Please use a different name.")); return; }
-			QListViewItem* se = new QListViewItem(m_engines, m_engine_name->text(), m_engine_url->text());
-			m_engines->insertItem(se);
-			m_items.append(se);
+			if ( !url.isValid() ) 
+			{ 
+				KMessageBox::error(this, i18n("Malformed URL.")); 
+				return; 
+			}
+			
+			if (m_engines->findItem(m_engine_name->text(), 0)) 
+			{
+				KMessageBox::error(this, i18n("A search engine with the same name already exists. Please use a different name.")); return; 
+			}
+			
+			new QListViewItem(m_engines, m_engine_name->text(), m_engine_url->text());
 			m_engine_url->setText("");
 			m_engine_name->setText("");
 		}
 		else
+		{
 			KMessageBox::error(this, i18n("Bad URL. You should search for FOOBAR with your Internet browser and copy/paste the exact URL here."));
+		}
 	}
  
 	void SearchPrefPageWidget::removeClicked()
 	{
-		QListView trt;
-		if ( m_engines->selectedItem() == 0 ) return;
-     
+		if ( m_engines->selectedItem() == 0 ) 
+			return;
+ 
 		QListViewItem* item = m_engines->selectedItem();
 		m_engines->takeItem(item);
-		m_items.remove(item);
+		delete item;
 	}
  
 	void SearchPrefPageWidget::addDefaultClicked()
 	{
 		QListViewItem* se = new QListViewItem(m_engines, "KTorrents", "http://www.ktorrents.com/search.php?lg=0&sourceid=ktorrent&q=FOOBAR&f=0");
-		m_items.append(se);
-		m_engines->insertItem(se);
 		
 		se = new QListViewItem(m_engines, "bittorrent.com", "http://search.bittorrent.com/search.jsp?query=FOOBAR");
-		m_items.append(se);
-		m_engines->insertItem(se);
      
 		se = new QListViewItem(m_engines, "isohunt.com", "http://isohunt.com/torrents.php?ihq=FOOBAR&op=and");
-		m_items.append(se);
-		m_engines->insertItem(se);
      
 		se = new QListViewItem(m_engines, "mininova.org", "http://www.mininova.org/search.php?search=FOOBAR");
-		m_items.append(se);
-		m_engines->insertItem(se);
      
 		se = new QListViewItem(m_engines, "thepiratebay.org", "http://thepiratebay.org/search.php?q=FOOBAR");
-		m_items.append(se);
-		m_engines->insertItem(se);
      
 		se = new QListViewItem(m_engines, "bitoogle.com", "http://search.bitoogle.com/search.php?q=FOOBAR&st=t");
-		m_items.append(se);
-		m_engines->insertItem(se);
      
 		se = new QListViewItem(m_engines, "bytenova.org", "http://www.bitenova.org/search.php?search=FOOBAR&start=0&start=0&ie=utf-8&oe=utf-8");
-		m_items.append(se);
-		m_engines->insertItem(se);
      
 		se = new QListViewItem(m_engines, "torrentspy.com", "http://torrentspy.com/search.asp?query=FOOBAR");
-		m_items.append(se);
-		m_engines->insertItem(se);
 
 		se = new QListViewItem(m_engines, "torrentz.com", "http://www.torrentz.com/search_FOOBAR");
-		m_items.append(se);
-		m_engines->insertItem(se);
 	}
  
 	void SearchPrefPageWidget::removeAllClicked()
 	{
 		m_engines->clear();
-		m_items.clear();
 	}
 	
 	void SearchPrefPageWidget::btnUpdate_clicked()
@@ -202,10 +188,9 @@ namespace kt
 		if (KIO::NetAccess::download(source,fn,NULL))
 		{
 			//list successfully downloaded, remove temporary file
-			KIO::NetAccess::removeTempFile(fn);
 			updateList(fn);
 			saveSearchEngines();
-			KIO::NetAccess::del(fn);
+			KIO::NetAccess::removeTempFile(fn);
 		}
 	}
 	
@@ -218,9 +203,8 @@ namespace kt
  
 		QTextStream in(&fptr);
 		
-		QStringList sNames;
-		QStringList sUrls;
-
+		QMap<QString,KURL> engines;
+		
 		while (!in.atEnd())
 		{
 			QString line = in.readLine();
@@ -236,34 +220,21 @@ namespace kt
 			for(Uint32 i=2; i<tokens.count(); ++i)
 				url.addQueryItem(tokens[i].section("=",0,0), tokens[i].section("=", 1, 1));
 			
-			sNames << name;
-			sUrls << url.url();
+			engines.insert(name,url);
 		}
 		
-		QStringList existing;
-		//modify
-		for(int i=0; i<m_items.count(); ++i)
+		QMap<QString,KURL>::iterator i = engines.begin();
+		while (i != engines.end())
 		{	
-			int index = sNames.findIndex(m_items.at(i)->text(0));
-			if(index != -1)
-			{
-				//replace
-				QListViewItem* se = m_items.at(i);
-				se->setText(1, sUrls[index]);
-				
-				sNames.remove(sNames.at(index));
-				sUrls.remove(sUrls.at(index));
-			}
+			QListViewItem* item = m_engines->findItem(i.key(),0);
+			// if we have found the item, replace it if not make a new one
+			if (item)
+				item->setText(1, i.data().prettyURL());
+			else
+				new QListViewItem(m_engines,i.key(),i.data().prettyURL());
+			
+			i++;
 		}
-		
-		int i=0;
-		for(QStringList::Iterator it = sNames.begin(); it != sNames.end(); ++it, ++i) 
-		{
-			QListViewItem* se = new QListViewItem(m_engines, *it, sUrls[i]);
-			m_items.append(se);
-			m_engines->insertItem(se);
-		}
-		
 	}
 	
 	////////////////////////////////////////////////////////////////////////////////////////////
@@ -302,7 +273,13 @@ namespace kt
 
 	void SearchPrefPage::updateData()
 	{
-		widget->loadSearchEngines();
+		widget->updateSearchEngines(m_plugin->getSearchEngineList());
+		
+	}
+	
+	void SearchPrefPageWidget::customToggled(bool toggled)
+	{
+		customBrowser->setEnabled(toggled);
 	}
 }
 
